/*
    Scanbot Barcode Scanner SDK Flutter Plugin

    Copyright (c) 2020 doo GmbH. All rights reserved.
    https://scanbot.io/sdk
 */
package io.scanbot.sdk.flutter

import android.util.Base64
import com.google.gson.Gson
import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonWriter
import io.scanbot.sdk.flutter.camera.config.BarcodeScannerConfiguration
import io.scanbot.sdk.flutter.camera.config.FinderConfig
import io.scanbot.sdk.flutter.camera.config.SelectionOverlayConfiguration
import org.json.JSONObject


typealias BarcodeScannerConfigurationJson = HashMap<String, Any>
typealias BarcodeDetectionParams = HashMap<String, Any>
typealias AspectRatioJson = HashMap<String, Any>
typealias BarcodeFormattedJson = HashMap<String, Any>
typealias Json = HashMap<String, Any>

// Using Android's base64 libraries. This can be replaced with any base64 library.
class ByteArrayToBase64TypeAdapter : TypeAdapter<ByteArray>() {
    override fun write(out: JsonWriter?, value: ByteArray?) {
        out?.value(value?.let { Base64.encodeToString(it, Base64.NO_WRAP) } ?: "")
    }

    override fun read(inner: JsonReader?): ByteArray {
        return ByteArray(0)// we do not have any byte arrays going in sdk
    }
}

fun Json.toFinderConfig(gson: Gson): FinderConfig {
    return gson.fromJson(JSONObject(this as Map<*, *>).toString(), FinderConfig::class.java)
}

fun Json.toBarcodeScannerConfig(gson: Gson): BarcodeScannerConfiguration {
    return gson.fromJson(JSONObject(this as Map<*, *>).toString(), BarcodeScannerConfiguration::class.java)
}

fun Json.toSelectionOverlayConfig(gson: Gson): SelectionOverlayConfiguration {
    return gson.fromJson(
        JSONObject(this as Map<*, *>).toString(),
        SelectionOverlayConfiguration::class.java
    )
}


fun <T> Map<String, Any>.checkParam(name: String, value: (T) -> Unit = {}): T? {
    return this.get(name)?.also {
        value(it as T)
    } as? T
}