package io.scanbot.sdk.flutter.camera.detectorWrappers

import com.google.gson.JsonObject
import io.flutter.plugin.common.MethodChannel
import io.scanbot.sdk.flutter.Json
import kotlinx.coroutines.suspendCancellableCoroutine

interface IDetectorWrapper {
    val gson: com.google.gson.Gson
    fun attach(arguments: Json)

    fun detach()

    fun toggleContinuousFocus()

    fun isFlashAvailable():Boolean

    fun flashEnabled(enabled: Boolean)

    suspend fun <T> callMethodSync(channel: MethodChannel, methodName:String, json: JsonObject) = suspendCancellableCoroutine {

        channel.invokeMethod(
            methodName,
            gson.toJson(json),
            object : MethodChannel.Result {
                override fun success(result: Any?) {
                    it.resume(result as? T, {})
                }

                override fun error(
                    errorCode: String,
                    errorMessage: String?,
                    errorDetails: Any?
                ) {
                    it.resume(null, {})
                }

                override fun notImplemented() {
                    it.resume(null, {})
                }
            }
        )
    }
}
