//
//  StorageHandler.swift
//  barcode_scanner
//
//  Created by Seifeddine Bouzid on 19.05.23.
//

import Foundation
import ScanbotBarcodeScannerSDK

class StorageHandler {
    static let `default` = StorageHandler()
    private var storageURL: URL?
    private var storageManager: FileManager
    
    private init() {
        storageURL = FileManager.default.urls(for: .cachesDirectory, in: .userDomainMask).first
        storageManager = FileManager.default
    }
    
    func addImage(_ image: UIImage) -> String? {
        let urlString = self.url(with: UUID().uuidString)
        if storageManager.createFile(atPath: urlString, contents: image.pngData()) {
            return urlString
        }
        return nil
    }
    
    private func url(with id: String) -> String {
        if let storageURL {
            if #available(iOS 16.0, *) {
                return storageURL.appending(path: id+".png").path()
            } else {
                // Fallback on earlier versions
                return storageURL.appendingPathComponent(id+".png").path
            }
        }
        return ""
    }
}
