import 'package:flutter/widgets.dart';
import 'package:json_annotation/json_annotation.dart';

import '../json/common.dart';
import '../json/common_platform.dart' as common;
import 'finder.dart';
export 'finder.dart';

part 'camera_configuration.g.dart';

abstract class CameraConfiguration {
  DetectorType get detectorType;

  bool handleCameraInAppBackgroundAutomatically = true;

  Map<String, dynamic> toJson();
}

enum DetectorType {
  barcode,
  mc,
  mrz,
  ehic,
  document,
  payform,
}

/// The finder view is a rectangular overlay view that clips the camera view, so that the detection will focus just a specific area.
/// You can specify the size and positioning, as well as other UI properties.
/// Set this property to `true` if you want to use the Finder View, `false` otherwise (default: false)
@JsonSerializable(
    createFactory: false, includeIfNull: false, explicitToJson: true)
class FinderConfiguration {
  /// The shape by which background will be clipped and which will be presented as finder hole
  @JsonKey(includeToJson: false)
  BoxDecoration decoration;

  /// This widget will be drawn inside of the finder hole.
  /// Here you can add some custom design, for example, with qr code placeholder
  @JsonKey(includeToJson: false)
  Widget? widget;

  /// Something that should be shown under the finder view window
  @JsonKey(includeToJson: false)
  Widget? topWidget;

  /// Something that should be shown below the finder view window
  @JsonKey(includeToJson: false)
  Widget? bottomWidget;

  /// The overlay background color, around the finder view rectangle
  @JsonKey(includeToJson: false)
  Color? backgroundColor;

  /// The finder view rectangle aspect ratio
  common.AspectRatio? finderAspectRatio =
      common.AspectRatio(width: 1, height: 1);

  /// The finder view Insets. You can use it to move the finder view rectangle around the screen and to set borders around it.
  Insets? finderInsets;

  /// Triggers when the finder area is changed, it's helpful to align some content on top of the finder
  @JsonKey(includeToJson: false)
  final OnFinderRectChange? onFinderRectChange;

  FinderConfiguration(
      {required this.decoration,
      this.backgroundColor,
      this.finderAspectRatio,
      this.widget,
      this.topWidget,
      this.bottomWidget,
      this.finderInsets =
          const Insets(left: 20, right: 20, bottom: 20, top: 20),
      this.onFinderRectChange});

  Map<String, dynamic> toJson() => _$FinderConfigurationToJson(this);
}
